/* extension.js
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*
* SPDX-License-Identifier: GPL-2.0-or-later
*/

import GObject from 'gi://GObject';
import St from 'gi://St';
const { Soup, GLib } = imports.gi;

import { Extension, gettext as _ } from 'resource:///org/gnome/shell/extensions/extension.js';
import * as PanelMenu from 'resource:///org/gnome/shell/ui/panelMenu.js';
import * as PopupMenu from 'resource:///org/gnome/shell/ui/popupMenu.js';

import * as Main from 'resource:///org/gnome/shell/ui/main.js';

const URI_PUBLIC_IP_V4 = "https://api.ipify.org";
const URI_PUBLIC_IP_V6 = "https://api6.ipify.org";
const newMenuItem = (label) => {
    return new PopupMenu.PopupMenuItem(label);
}

const Indicator = GObject.registerClass(
    class Indicator extends PanelMenu.Button {

        fetchIPs(uri) {
            return new Promise(async (resolve, reject) => {
                try{
                    const message = Soup.Message.new('GET', uri);
                    const session = new Soup.Session();
                    const bytes = await session.send_and_read_async(
                        message,
                        GLib.PRIORITY_DEFAULT,
                        null,
                    );

                    if (message.get_status() != Soup.Status.OK) {
                        console.log(`HTTP Status ${message.status_code}`);
                    }

                    const textDecoder = new TextDecoder("utf-8");
                    const publicIP = textDecoder.decode(bytes.toArray());
                    Main.notify(_('Whatʼs up? your ip is ' + publicIP));
                    console.log("Response: " + publicIP);
                    resolve(publicIP);
                } catch(e) {
                    reject(e);
                }

            });
        }


        _init(extension) {
            super._init(0.0, _('MyPubIP'));
            this.extension = extension;

            this.add_child(new St.Icon({
                icon_name: 'network-server-symbolic',
                style_class: 'system-status-icon',
            }));
            // refresh button
            this.refreshItem = newMenuItem('Refresh');
            this.refreshItem.connect('activate', async () => {
                this.fetchIPs(this.itemIPv4);
                this.fetchIPs(this.itemIPv6);
            });
            this.menu.addMenuItem(this.refreshItem);
            this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());

            // for api v4
            this.fetchIPs(URI_PUBLIC_IP_V4).then((ipv4str) => {
                this.ipv4str = ipv4str;
                console.log(ipv4str);
                let itemIPv4 = newMenuItem(ipv4str);
                itemIPv4.connect('activate', async () => {
                    console.log("Response: " + this.ipv4str);
                    Main.notify(_(`Copy IPv4: ${this.ipv4str}`));
                    
                })
                this.menu.addMenuItem(itemIPv4);
            }).catch(e => {
                console.log("Error: could not fetch public IP address: " + e);
            });

            // for api v6
            this.fetchIPs(URI_PUBLIC_IP_V6).then((ipv6str) => {
                this.ipv6str = ipv6str;
                console.log(ipv6str);
                let itemIPv6 = newMenuItem(ipv6str);
                itemIPv6.connect('activate', async () => {
                    console.log("Response: " + this.ipv6str);
                    Main.notify(_(`Copy IPv6: ${this.ipv6str}`));
                    St.Clipboard
                })
                this.menu.addMenuItem(itemIPv6);
            }).catch(e => {
                console.log("Error: could not fetch public IP address: " + e);
            });   
        }
    });

export default class MyPubIP extends Extension {
    enable() {
        this._indicator = new Indicator();
        Main.panel.addToStatusArea(this.uuid, this._indicator);
    }

    disable() {
        this._indicator.destroy();
        this._indicator = null;
    }
}
